@@ -390,6 +390,22 @@ writefile(time_t runtimer, char queue)
 	int export = 1;
 	char *eqp;
 
+        /* Only accept alphanumerics and underscore in variable names.
+         * Also require the name to not start with a digit.
+         * Some shells don't like other variable names.
+         */
+        {
+            char *p = *atenv;
+            if (isdigit(*p))
+                export = 0;
+            for (; *p != '=' && *p != '\0'; ++p) {
+                if (!isalnum(*p) && *p != '_') {
+                    export = 0;
+                    break;
+                }
+            }
+        }
+
 	eqp = strchr(*atenv, '=');
 	if (ap == NULL)
 	    eqp = *atenv;
@@ -477,6 +493,9 @@ writefile(time_t runtimer, char queue)
 	fprintf(stderr, "<EOT>\n");
     }
     fprintf(fp, "\n");
+    if (ferror(fp))
+	panic("Output error");
+    fflush(fp);
     if (ferror(fp))
 	panic("Output error");
 
@@ -730,7 +749,7 @@ main(int argc, char **argv)
     char *pgm;
 
     int program = AT;		/* our default program */
-    char *options = "q:f:MmvlrdhVct:";	/* default options for at */
+    char *options = "q:f:MmbvlrdhVct:";	/* default options for at */
     int disp_version = 0;
     time_t timer = 0;
     struct passwd *pwe;
